Package: vDiveR
Type: Package
Title: Visualization of Viral Protein Sequence Diversity Dynamics
Version: 2.2.1
Authors@R: c(
    person("Pendy", "Tok", email = "pendytok0518@gmail.com", role = c("aut", "cre")),
    person("Li Chuin", "Chong", role = "aut"),
    person("Evgenia", "Chikina", role = "aut"),
    person("Yin Cheng", "Chen", role = "aut"),
    person("Mohammad Asif", "Khan", role = "aut"))
Description: To ease the visualization of outputs from Diversity Motif Analyser ('DiMA';
    <https://github.com/BVU-BILSAB/DiMA>). 'vDiveR' allows visualization of the diversity 
    motifs (index and its variants – major, minor and unique) for elucidation of 
    the underlying inherent dynamics. Please refer <https://vdiver-manual.readthedocs.io/en/latest/>
    for more information.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: dplyr, ggplot2, ggpubr, grid, ggtext, magrittr, tidyr,
        stringr, rlang, rentrez, scales, utils, maps, cowplot,
        stringdist, patchwork
RoxygenNote: 7.3.3
Depends: R (>= 2.10)
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-23 10:51:21 UTC; pendy
Author: Pendy Tok [aut, cre],
  Li Chuin Chong [aut],
  Evgenia Chikina [aut],
  Yin Cheng Chen [aut],
  Mohammad Asif Khan [aut]
Maintainer: Pendy Tok <pendytok0518@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-23 12:40:02 UTC
Built: R 4.6.0; ; 2026-07-23 14:39:25 UTC; unix

# Generated by roxygen2: do not edit by hand

export(concat_conserved_kmer)
export(json2csv)
export(metadata_extraction)
export(plot_conservation_level)
export(plot_correlation)
export(plot_dynamics_protein)
export(plot_dynamics_proteome)
export(plot_entropy)
export(plot_time)
export(plot_world_map)
importFrom(cowplot,draw_label)
importFrom(cowplot,draw_plot)
importFrom(cowplot,ggdraw)
importFrom(cowplot,plot_grid)
importFrom(dplyr,"%>%")
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,right_join)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(dplyr,vars)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,map_data)
importFrom(ggplot2,margin)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_color_grey)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_date)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,sec_axis)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,vars)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(ggpubr,annotate_figure)
importFrom(ggpubr,ggarrange)
importFrom(ggpubr,text_grob)
importFrom(ggtext,geom_richtext)
importFrom(grid,unit)
importFrom(magrittr,"%>%")
importFrom(maps,map)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(rentrez,entrez_fetch)
importFrom(rentrez,entrez_search)
importFrom(rlang,":=")
importFrom(scales,date_format)
importFrom(scales,math_format)
importFrom(scales,trans_breaks)
importFrom(scales,trans_format)
importFrom(stats,aggregate)
importFrom(stringdist,stringdist)
importFrom(stringr,str_extract)
importFrom(stringr,str_sub)
importFrom(stringr,str_to_title)
importFrom(tidyr,replace_na)
importFrom(tidyr,separate)
importFrom(tidyr,spread)

# vDiveR

<!-- badges: start -->

[![CRAN status](https://www.r-pkg.org/badges/version/vDiveR)](https://CRAN.R-project.org/package=vDiveR)
[![R-CMD-check](https://github.com/pendy05/vDiveR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/pendy05/vDiveR/actions/workflows/R-CMD-check.yaml)

<!-- badges: end -->

To ease the visualization of outputs from Diversity Motif Analyser ('DiMA';
<https://github.com/BVU-BILSAB/DiMA>). 'vDiveR' allows visualization of the diversity
motifs (index and its variants – major, minor and unique) for elucidation of
the underlying inherent dynamics. Please refer <https://vdiver-manual.readthedocs.io/en/latest/>
for more information.

## Installation

```r
# install vDiveR:
install.packages("vDiveR")

#  Or the development version from GitHub:
# install.packages("devtools")
devtools::install_github("pendy05/vDiveR")

```

## VDiveR Functions Overview

<p align="center">
<img src="man/figures/vdiver_overview_chart.png" alt="overview on vDiveR functions." height="700"/>
</p>
Figure 1: vDiveR functions. In the image above, purpose of each function together with its main arguments is shown. The dataframe returned from the respective functions are colored in orange.

#' JSON2CSV
#'
#' This function converts DiMA (v5.0.9) JSON output file to a dataframe with 17
#' predefined columns which further acts as the input for other functions provided in this vDiveR package.
#'
#' @param json_data DiMA JSON output dataframe
#' @param host_name name of the host species
#' @param protein_name name of the protein
#' @return A dataframe which acts as input for the other functions in vDiveR package
#' @examples inputdf<-json2csv(JSON_sample)
#' @importFrom stats aggregate
#' @importFrom dplyr mutate_if right_join distinct
#' @importFrom tidyr replace_na
#' @export
json2csv <-function(json_data, host_name="unknown host", protein_name="unknown protein"){
    # Declare global variables to avoid R CMD check warnings
    Group.2 <- x <- results.position <- results.diversity_motifs <- motif_short <- NULL
    position <- entropy <- support <- low_support <- distinct_variants_incidence <- NULL
    total_variants_incidence <- Ma <- Mi <- U <- I <- proteinName <- NULL
    results.support <- results.low_support <- results.entropy <- NULL
    results.total_variants_incidence <- results.distinct_variants_incidence <- NULL
    average_entropy <- highest_entropy.position <- highest_entropy.entropy <- host <- NULL
    sequence <- multiIndex <- NULL

    data_flatten <- as.data.frame(json_data) %>%
        tidyr::unnest(cols = c(results.diversity_motifs))

    # data transformation
    motifs_incidence <- aggregate(
        data_flatten$incidence,
        list(data_flatten$results.position,data_flatten$motif_short),
        FUN=sum
    ) %>%
        tidyr::spread(Group.2,x) %>%                           # transpose the rows (motif-long & incidence) to columns (index, major, minor, unique)
        dplyr::mutate_if(is.numeric, ~(tidyr::replace_na(., 0)))      # replace NAN with 0
    #rename column 'Group.1' to 'results.position'


    colnames(motifs_incidence)[colnames(motifs_incidence) == 'Group.1'] <- "results.position"

    #sum the number of Index motif found in each position (if > 1 => multiIndex == TRUE)
    multiIndex<-data_flatten%>%
        dplyr::group_by(results.position) %>%
        dplyr::summarize(multiIndex = sum(motif_short=="I"), .groups = "drop") %>%
        as.data.frame()

    #merge multiIndex to motifs_incidence df
    motifs_incidence <-dplyr::left_join(motifs_incidence,multiIndex, by='results.position')%>%
        distinct()

    #replace multiIndex with boolean: x > 1 = TRUE and vice versa
    motifs_incidence$multiIndex <- ifelse(
        tidyr::replace_na(motifs_incidence$multiIndex, 0) > 1,
        TRUE,
        FALSE
    )

    # Ensure expected incidence columns exist even if absent in data
    for (col in c("I", "Ma", "Mi", "U")) {
        if (!col %in% colnames(motifs_incidence)) {
            motifs_incidence[[col]] <- 0
        }
    }

    # ----- Extract index sequence only (for positions that have Index motif)
    # extract the first encountered index kmer sequence if > 1 index is encountered for each position (rarely happens)
    index_sequences<-subset(data_flatten, motif_short == "I") %>% #extract rows that are of index
        dplyr::group_by(results.position) %>% #group them based on position
        dplyr::slice(1) %>% #take the first index encountered per position
        dplyr::ungroup() %>%
        dplyr::select(results.position, sequence) %>%
        as.data.frame() #return the data in dataframe

    # ------ Extract position-level fields from results (not motif-specific)
    results_df <- as.data.frame(json_data$results)

    results_keep <- results_df %>%
        dplyr::transmute(
            results.position = position,
            results.entropy = entropy,
            results.support = support,
            results.low_support = low_support,
            results.distinct_variants_incidence = distinct_variants_incidence,
            results.total_variants_incidence = total_variants_incidence
        )
    
    #replace low_support of NA to FALSE
    results_keep$results.low_support[is.na(results_keep$results.low_support)] <- FALSE

    # ---- Global highest/average entropy (same for all rows) ----
    highest_pos <- if (!is.null(json_data$highest_entropy$position)) json_data$highest_entropy$position else NA
    highest_ent <- if (!is.null(json_data$highest_entropy$entropy)) json_data$highest_entropy$entropy else NA
    avg_ent <- if (!is.null(json_data$average_entropy)) json_data$average_entropy else NA

    results_keep$highest_entropy.position <- highest_pos
    results_keep$highest_entropy.entropy <- highest_ent
    results_keep$average_entropy <- avg_ent

    # ---- all positions in results
    motifs <- dplyr::left_join(results_keep, motifs_incidence, by = "results.position")
    # If any positions had no motifs rows at all (rare), fill incidence NAs with 0
    motifs <- motifs %>%
        dplyr::mutate(
            I = tidyr::replace_na(I, 0),
            Ma = tidyr::replace_na(Ma, 0),
            Mi = tidyr::replace_na(Mi, 0),
            U = tidyr::replace_na(U, 0),
            multiIndex = tidyr::replace_na(multiIndex, FALSE)
        )

    # Join index sequence (will be NA for positions without Index motif)
    if (nrow(index_sequences) > 0) {
        motifs <- dplyr::left_join(
            motifs,
            index_sequences,
            by = "results.position"
        )
    } else {
        # Create empty column if no Index exists anywhere (edge-case)
        motifs$sequence <- NA
    }

    #assign host + protein name
    motifs['host'] <- host_name
    motifs['proteinName'] <- protein_name

    # ---- final rename
    motifs <- motifs %>%
        dplyr::transmute(
            proteinName = proteinName,
            position = results.position,
            count = results.support,              # keeping your original naming
            lowSupport = results.low_support,
            entropy = results.entropy,
            indexSequence = sequence,
            index.incidence = I,
            major.incidence = Ma,
            minor.incidence = Mi,
            unique.incidence = U,
            totalVariants.incidence = results.total_variants_incidence,
            distinctVariant.incidence = results.distinct_variants_incidence,
            multiIndex = multiIndex,
            averageEntropy = average_entropy,
            highestEntropy.position = highest_entropy.position,
            highestEntropy = highest_entropy.entropy,
            host = host
        )

    motifs

}

test_that("Test Case 1: json2csv works on sample data (JSON_sample), check df shape and column names",{
    csv_df <- json2csv(JSON_sample, protein_name="A")
    expect_equal(unique(csv_df$proteinName), "A")
    expect_equal(nrow(csv_df),183)
    expect_equal(ncol(csv_df),17)

    col_names<-colnames(csv_df)
    expect_equal(col_names[1],'proteinName')
    expect_equal(col_names[2],'position')
    expect_equal(col_names[3],'count')
    expect_equal(col_names[4],'lowSupport')
    expect_equal(col_names[5],'entropy')
    expect_equal(col_names[6],'indexSequence')
    expect_equal(col_names[7],'index.incidence')
    expect_equal(col_names[8],'major.incidence')
    expect_equal(col_names[9],'minor.incidence')
    expect_equal(col_names[10],'unique.incidence')
    expect_equal(col_names[11],'totalVariants.incidence')
    expect_equal(col_names[12],'distinctVariant.incidence')
    expect_equal(col_names[13],'multiIndex')
    expect_equal(col_names[14],'averageEntropy')
    expect_equal(col_names[15],'highestEntropy.position')
    expect_equal(col_names[16],'highestEntropy')
    expect_equal(col_names[17],'host')


})

test_that("Test Case 2: json2csv works on sample data (JSON_sample), check data integrity",{
    csv_df <- json2csv(JSON_sample, protein_name="A")

    expect_equal(unique(csv_df$highestEntropy.position),66)
    expect_equal(unique(csv_df$highestEntropy),5.1592951)
    expect_equal(unique(csv_df$averageEntropy),1.81526603)
    expect_equal(unique(csv_df$host), 'unknown host')

    fifthRow <- csv_df[5,]
    expect_equal(fifthRow$proteinName, 'A')
    expect_equal(fifthRow$position, 5)
    expect_equal(fifthRow$count, 4424)
    expect_equal(fifthRow$lowSupport, FALSE)

    expect_equal(fifthRow$entropy, 1.9135753)
    expect_equal(fifthRow$indexSequence, 'PKPQRKTKR')
    expect_equal(fifthRow$index.incidence, 76.243225)
    expect_equal(fifthRow$major.incidence, 7.617541)
    expect_equal(fifthRow$minor.incidence, 13.743219)
    expect_equal(fifthRow$unique.incidence, 2.3960219)

    expect_equal(fifthRow$totalVariants.incidence, 23.75678)
    expect_equal(fifthRow$distinctVariant.incidence, 16.745956)
    expect_equal(fifthRow$multiIndex, FALSE)
})

#' Entropy plot
#'
#' This function plot entropy (black) and total variant (red) incidence of each
#' k-mer position across the studied proteins and highlight region(s) with zero entropy in yellow.
#' k-mer position with low support is marked with a red triangle underneath the x-axis line.
#'
#' @param df DiMA JSON converted csv file data
#' @param host number of host (1/2)
#' @param protein_order order of proteins displayed in plot
#' @param kmer_size size of the k-mer window
#' @param ymax maximum y-axis
#' @param line_size size of the horizontal (reference) line in plot
#' @param base_size word size in plot
#' @param all plot both the entropy and total variants (pass FALSE in to plot only the entropy)
#' @param highlight_zero_entropy highlight region with zero entropy (default: TRUE)
#' @return A plot
#' @examples plot_entropy(proteins_1host)
#' @examples plot_entropy(protein_2hosts, host = 2)
#' @importFrom ggplot2 geom_rect geom_area geom_hline geom_line facet_grid
#' @importFrom ggplot2 sec_axis element_line scale_colour_manual scale_linetype_manual
#' @export
plot_entropy <- function(df,
                         host=1,
                         protein_order="",
                         kmer_size=9,
                         ymax = 10,
                         line_size=2,
                         base_size=8,
                         all= TRUE,
                         highlight_zero_entropy=TRUE){
    entropy <- end <- lowSupportPos <- totalVariants.incidence <- NULL

    df$proteinName <- toupper(df$proteinName)

    #determine number of host
    #scale the amino acid position for each protein
    if (host ==1){ #single host
        if (protein_order =="" || is.null(protein_order) || protein_order == "NULL"){
            a<-table(df$proteinName)
            proteinName<-as.vector(names(a))
            position<-as.vector(a)
            df$size_f = factor(df$proteinName,levels = proteinName)
            scales_x<-mapply(function(x,y){
                x = scale_x_continuous(limits = c(0,y),breaks = seq(0,y,50))
            }, proteinName,position)
        }else{
            #order the proteins based on user input
            level<-strsplit(protein_order, ',')[[1]]
            level<- sapply(level, function(x) toupper(trimws(x)))
            position<-c()
            for (i in level){
                position<-append(position,table(df$proteinName)[names(table(df$proteinName)) == i])
            }
            df$size_f = factor(df$proteinName,levels = level)
            scales_x<-mapply(function(x,y){
                x = scale_x_continuous(limits = c(0,y),breaks = seq(0,y,50))
            }, level,position)
        }
    }else{ # multihost
        #categorise data based on host
        df$host<- factor(df$host)
        #count the aa length for each proteins (each host is expected to have same number of proteins with same length)
        df_sub<-df[df$host==unique(df$host[1]),]
        if (protein_order =="" || is.null(protein_order) || protein_order == "NULL"){
            a<-table(df_sub$proteinName)
            proteinName<-as.vector(names(a))
            position<-as.vector(a)
            df$size_f = factor(df$proteinName,levels = proteinName)
            scales_x<-mapply(function(x,y){
                x = scale_x_continuous(limits = c(0,y),breaks = seq(0,y,50))
            }, proteinName,position)
        }else{
            #order the proteins based on user input
            level<-strsplit(protein_order, ',')[[1]]
            level<- sapply(level, function(x) toupper(trimws(x)))
            position<-c()
            for (i in level){
                position<-append(position,table(df_sub$proteinName)[names(table(df_sub$proteinName)) == i])
            }
            df$size_f = factor(df$proteinName,levels = level)
            scales_x<-mapply(function(x,y){
                x = scale_x_continuous(limits = c(0,y),breaks = seq(0,y,50))
            }, level,position)
        }
    }

    #---------------set zero entropy region-----------------#
    #check if the kmer positions are zero entropy
    #Reference for getting all the positions of kmer based on kmer size and the starting position of kmer:
    #https://stackoverflow.com/questions/31120552/generate-a-sequence-of-numbers-with-repeated-intervals
    #https://stackoverflow.com/questions/41637518/adding-a-shaded-rectangle-to-an-existing-plot-using-geom-rect

    if(highlight_zero_entropy){
        #identify the kmer position with zero entropy
        #position: startPosition with zero entropy
        #end: startPosition + kmersize - 1
        df_zeroEntropy<- df %>%
            dplyr::group_by(proteinName)%>%
            dplyr::reframe(
                end = position[which(entropy == min(entropy))+kmer_size-1], #end: startPosition + kmersize - 1
                position = position[which(entropy == min(entropy))] #extract those starting positions with zero entropy
            )%>%
            as.data.frame()

        #concatenate df with df_zeroEntropy
        df <-merge(df,df_zeroEntropy,id="position",all=T)
        #replace NAN in column "zero entropy" with FALSE
        df$end[is.na(df$end)]<- -1
    }

    #---------------set maximum y limit-----------------#
    #if the max y value set by user is less than the max y value in data, then set the max y value to the max y value in data
    if (ymax < max(df$entropy) && max(df$entropy) > 10){
       ymax <- ceiling(max(df$entropy))
    } else if (ymax < 10){ # to ensure the reference hline is visible
        ymax <- 10
    }

    breaks_fun <- function(x) {
        max_pos <- max(x)
        # Calculate adaptive interval to aim for approximately 5-7 tick marks
        if (max_pos <= 50) {
            interval <- 5
        } else if (max_pos <= 100) {
            interval <- 10
        } else if (max_pos <= 200) {
            interval <- 20
        }  else if (max_pos <= 500) {
            interval <- 50
        } else if (max_pos <= 1000) {
            interval <- 100
        } else if (max_pos <= 2000) {
            interval <- 200
        } else {
            interval <- 250
        }
        seq(0, max_pos, interval)
    }

    limits_fun <- function(x) {
        c(0,max(x))
    }

    #----------------plotting--------------------#
    df$lowSupportPos <- -0.3
    df$lowSupportPos[df$lowSupport ==TRUE]<- -0.5

    plot1<-ggplot(df) +
        geom_area(mapping = aes(x = position, y = entropy,color= "k-mer Entropy", linetype="k-mer Entropy"), show.legend=T)+
        geom_hline(mapping = aes(yintercept=9.2, color = "Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)", linetype = "Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)"), linewidth= (line_size/10))+
        labs(y = "k-mer entropy (bits)\n",x= "\nk-mer position (aa)",color = "#f7238a")+
        scale_x_continuous(limits = limits_fun,breaks = breaks_fun)+
        theme_classic(base_size = base_size) +
        theme(
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            axis.line.y.right = element_line(color = "#f7238a"),
            axis.ticks.y.right = element_line(color = "#f7238a"),
            axis.title.y.right =  element_text(color = "#f7238a"),
            legend.position="bottom"
        )+
        scale_colour_manual("",
                            values = c("Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)"="black",
                                       "k-mer Entropy" = "black"))+
        scale_linetype_manual("",values=c("Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)"=5,
                                          "k-mer Entropy" = 1)) +
        ylim(0, ymax)+
        guides(color = guide_legend(nrow = 2),
               linetype = guide_legend(nrow = 2))


    # detect if low support present
    if (TRUE %in% df$lowSupport){
        plot1 <- plot1 +
            geom_point(mapping = aes(x = position,y=lowSupportPos),col=ifelse(df$lowSupportPos==-0.5, 'black', ifelse(df$lowSupportPos==-0.3, 'white', 'white')), alpha=ifelse(df$lowSupportPos==-0.5, 1, ifelse(df$lowSupportPos==-0.3, 0,0)),pch=17)
    }


    # highlight region with zero entropy
    if(highlight_zero_entropy){
        plot1<- plot1+
            geom_rect(inherit.aes = FALSE,
                      aes(xmin=position, xmax=end, ymin=-Inf, ymax=+Inf, color = "Zero Entropy",linetype="Zero Entropy"),
                      fill='#FFECAF', alpha=ifelse(df$end == -1, 0, 0.5))
    }

    # plot both entropy and total variants
    if(all){
        # Prepare legend breaks and values based on highlight_zero_entropy
        if(highlight_zero_entropy){
            legend_breaks <- c(
                "k-mer Entropy",
                "Total Variants",
                "Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)",
                "Reference: Maximum Total Variants (98%) for HIV-1 Clade B (Env Protein)",
                "Zero Entropy"
            )
            color_values <- c("Reference: Maximum Total Variants (98%) for HIV-1 Clade B (Env Protein)"="#f7238a",
                             "Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)"="black",
                             "k-mer Entropy" = "black",
                             "Total Variants"="#f7238a",
                             "Zero Entropy" = '#FFECAF')
            linetype_values <- c("Reference: Maximum Total Variants (98%) for HIV-1 Clade B (Env Protein)"=5,
                                "Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)"=5,
                                "k-mer Entropy" = 1,
                                "Total Variants"=1,
                                "Zero Entropy" = 1)
            fill_override <- c("black", NA, NA, NA, '#FFECAF')
        } else {
            legend_breaks <- c(
                "k-mer Entropy",
                "Total Variants",
                "Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)",
                "Reference: Maximum Total Variants (98%) for HIV-1 Clade B (Env Protein)"
            )
            color_values <- c("Reference: Maximum Total Variants (98%) for HIV-1 Clade B (Env Protein)"="#f7238a",
                             "Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)"="black",
                             "k-mer Entropy" = "black",
                             "Total Variants"="#f7238a")
            linetype_values <- c("Reference: Maximum Total Variants (98%) for HIV-1 Clade B (Env Protein)"=5,
                                "Reference: Maximum Entropy (9.2) for HIV-1 Clade B (Env Protein)"=5,
                                "k-mer Entropy" = 1,
                                "Total Variants"=1)
            fill_override <- c("black", NA, NA, NA)
        }
        
        plot1<- plot1 + geom_line(mapping = aes(x = position, y = totalVariants.incidence * ymax / 100, color = "Total Variants",linetype="Total Variants"), linewidth= (line_size/10) )+
        geom_hline(mapping = aes( yintercept=98* ymax / 100, color = "Reference: Maximum Total Variants (98%) for HIV-1 Clade B (Env Protein)",linetype ="Reference: Maximum Total Variants (98%) for HIV-1 Clade B (Env Protein)"), linewidth= (line_size/10))+
        #how to second y-axis: https://whatalnk.github.io/r-tips/ggplot2-rbind.nb.html
        scale_y_continuous(sec.axis = sec_axis(~ . * 100 / ymax , name = "Total variants (%)",breaks = c(0,25,50,75,100),labels=c("0","25","50","75","100")),
                            breaks = seq(0.0, ymax, length.out = 5),labels= sprintf(seq(0.0, ymax, length.out = 5), fmt = "%.1f")) +
        scale_colour_manual("",
                            values = color_values,
                            breaks = legend_breaks)+
        scale_linetype_manual("",
                              values = linetype_values,
                              breaks = legend_breaks)+
        guides(color = guide_legend(nrow = 2, override.aes = list(fill = fill_override)),
               linetype = guide_legend(nrow = 2))
    }

    #number of host
    if (host == 1){ #one host
        plot1 +facet_grid(cols=vars(df$size_f),scales = 'free',space = "free",switch = "x")
    }else{ # multi host
        plot1 +facet_grid(rows = vars(df$host),cols=vars(df$size_f),scales = 'free',space = "free",switch = "both")
    }

}

#' Conservation Levels Distribution Plot
#'
#' This function plots conservation levels distribution of k-mer positions, which consists of
#' completely conserved (black) (index incidence = 100\%), highly conserved (blue)
#' (90\% <= index incidence < 100\%), mixed variable (green) (20\% < index incidence <= 90\%),
#' highly diverse (purple) (10\% < index incidence <= 20\%) and
#' extremely diverse (pink) (index incidence <= 10\%).
#'
#' @param df DiMA JSON converted csv file data
#' @param protein_order order of proteins displayed in plot
#' @param conservation_label 0 (partial; show present conservation labels only) or 1 (full; show ALL conservation labels) in plot
#' @param host number of host (1/2)
#' @param base_size base font size in plot
#' @param line_dot_size lines and dots size
#' @param label_size conservation labels font size
#' @param alpha any number from 0 (transparent) to 1 (opaque)
#' @examples plot_conservation_level(proteins_1host, conservation_label = 1,alpha=0.8, base_size = 15)
#' @examples plot_conservation_level(protein_2hosts, conservation_label = 0, host=2)
#' @return A plot
#' @importFrom dplyr case_when
#' @importFrom grid unit
#' @importFrom cowplot plot_grid
#' @export
plot_conservation_level <- function(df,
                                   protein_order=NULL,
                                   conservation_label=1,
                                   host=1,
                                   base_size = 11,
                                   line_dot_size = 2,
                                   label_size = 2.6,
                                   alpha=0.6){
    df <- df %>% data.frame() %>%
        dplyr::mutate(
        ConservationLevel = case_when(
            index.incidence == 100 ~ "Completely conserved (CC)",
            index.incidence >= 90 ~ "Highly conserved (HC)",
            index.incidence >= 20 ~ "Mixed variable (MV)",
            index.incidence >= 10  ~ "Highly diverse (HD)",
            index.incidence < 10 ~ "Extremely diverse (ED)"
        )
        )

    #single host
    if (host == 1){
        plot_plot7(data = df,protein_order = protein_order,conservation_label = conservation_label, base_size = base_size, label_size = label_size, line_dot_size = line_dot_size, alpha = alpha)
    }else{ #multihost

        #split the data into multiple subsets (if multiple hosts detected)
        plot7_list<-split(df,df$host)
        plot7_multihost<-lapply(plot7_list, plot_plot7,
                                 protein_order = protein_order,
                                 conservation_label = conservation_label,
                                 base_size = base_size,
                                 line_dot_size = line_dot_size,
                                 label_size = label_size,
                                 alpha = alpha)

        # Remove legends from each individual plot
        plot7_no_legend <- lapply(plot7_multihost, function(p) p + theme(legend.position = "none"))
        # Combine the plots and add a single legend at the bottom
        combined_plot <- plot_grid(plotlist = plot7_no_legend,
                                ncol = 1)
        # Extract the legend from one of the plots (assuming all plots have the same legend)
        shared_legend <- cowplot::get_plot_component(plot7_multihost[[1]], "guide-box", return_all = TRUE)[[3]]
        # Combine the grid with the legend at the bottom
        plot_grid(combined_plot, shared_legend, ncol = 1, nrow = length(unique(df$host)), rel_heights = c(1, .1))
    }
}


#' @importFrom ggplot2 position_jitter scale_colour_manual scale_x_continuous
#' @importFrom ggplot2 position_dodge coord_cartesian geom_jitter
#' @importFrom ggtext geom_richtext
#plotting function
plot_plot7<- function(data,
                      protein_order=NULL,
                      conservation_label=1,
                      base_size = 11,
                      line_dot_size = 2,
                      label_size = 2.6,
                      alpha =0.6){
    proteinName <- Total <- index.incidence <- NULL
    label <- ConservationLevel <- level_data <- x_pos <- NULL
    C_level<- c("Completely conserved (CC)",
                "Highly conserved (HC)",
                "Mixed variable (MV)",
                "Highly diverse (HD)",
                "Extremely diverse (ED)")


    #add word 'protein' in front of each protein name
    data$proteinName <- toupper(data$proteinName)
    data$proteinName<-paste("Protein",data$proteinName)
    #create data for proteome bar "ALL" from existing data
    data1<-data
    data1$proteinName <- "ALL"
    data1$level <- "ALL"
    #set up the order of proteins in plot from left to right
    if (is.null(protein_order) || protein_order ==""){ #follow the default order in csv file
        level<-c("ALL",unique(data$proteinName))
    }else{ #order the proteins based on user input
        protein_order<-toupper(protein_order)
        level<-c("ALL",paste("Protein",trimws(strsplit(protein_order, ',')[[1]])))
    }

    #determine the protein order
    data$level = factor(data$proteinName, levels=level)
    #combine proteome bar with protein bars
    data<-rbind(data1,data)
    #determine the protein order
    data$level = factor(data$proteinName, levels=level)

    #--- calculation for total and percentage of conservation levels for each protein ----
    #sum up the total positions for each conservation level of proteins
    plot7_data <- data %>%
        dplyr::group_by(proteinName, ConservationLevel) %>%
        dplyr::summarise(count = n()) %>%
        dplyr::ungroup()
    names(plot7_data)[3]<-"Total"

    #check the presence of conservation level: insert value 0 if it is absent
    if (conservation_label == 1){ #full label
        #check the presence of conservation level: insert value 0 if it is absent
        for ( conservation in C_level){ #conservation level
            for (name in level){ #proteinName
                if (!(conservation %in% plot7_data[plot7_data$proteinName==name,]$ConservationLevel)){
                    plot7_data<-rbind(plot7_data,c(name,conservation,0))
                }}}
    }

    #sort the dataframe
    plot7_data[order(plot7_data$proteinName),]
    plot7_data$Total<- as.integer(plot7_data$Total)
    #get the percentage of each conservation level for each protein
    plot7_data <- plot7_data %>%
        dplyr::group_by(proteinName) %>%
        dplyr::mutate(percent = Total / sum(Total) * 100) %>%
        dplyr::ungroup()

    #gather the protein label in multicolor
    plot7_data<-plot7_data%>%mutate(label = case_when(
        plot7_data$ConservationLevel == "Completely conserved (CC)" ~ paste0(sprintf("<span style =
    'color:#000000;'>CC: %.0f (%.1f %%) </span>",plot7_data$Total, round(plot7_data$percent,1))),
        plot7_data$ConservationLevel == "Highly conserved (HC)" ~ paste0(sprintf("<span style =
    'color:#0057d1;'>HC: %.0f (%.1f %%) </span>",plot7_data$Total, round(plot7_data$percent,1))),
        plot7_data$ConservationLevel == "Mixed variable (MV)" ~ paste0(sprintf("<span style =
    'color:#02d57f;'>MV: %.0f (%.1f %%) </span>",plot7_data$Total, round(plot7_data$percent,1))),
        plot7_data$ConservationLevel == "Highly diverse (HD)" ~ paste0(sprintf("<span style =
    'color:#A022FF;'>HD: %.0f (%.1f %%) </span>",plot7_data$Total, round(plot7_data$percent,1))),
        plot7_data$ConservationLevel == "Extremely diverse (ED)" ~ paste0(sprintf("<span style =
    'color:#ff617d;'>ED: %.0f (%.1f %%) </span>",plot7_data$Total, round(plot7_data$percent,1))),
    ))

    # set conservation level in specific order (CC,HC,MV,HD,ED)
    # Ensure 'conservation' is a factor with all the levels
    plot7_data$ConservationLevel <- factor(plot7_data$ConservationLevel,
                                        levels=C_level)
    plot7_data<-plot7_data[order(plot7_data$ConservationLevel),]

    #combine all conservation level labels into one for each protein
    protein_labels<- aggregate(label~proteinName, plot7_data, paste, collapse="<br>")
    #get number of protein for labelling
    nProtein<-nrow(protein_labels)

    #--- append 5 dummy rows with each represents one conservation level ----
    # to ensure all 5 levels are shown in legend
    # level_data = 1 for protein data to be plotted, 0 for dummy data
    data$level_data <- 1
    first_row <- data[1, ]

    # Replicate the first row 5 times, each representing a different level
    new_rows <- do.call(rbind, replicate(5, first_row, simplify = FALSE))
    new_rows$ConservationLevel <- factor(C_level)
    new_rows$level_data <- 0
    # Append the new rows to the original data
    data <- rbind(data, new_rows)
    data$ConservationLevel <- factor(data$ConservationLevel)

    data$x_pos <- as.numeric(data$level)
    protein_labels$x_pos <- match(protein_labels$proteinName, level)

    #--- plotting ----
    ggplot(data %>% filter(level_data == 1) , aes(y=index.incidence)) +
        ggplot2::geom_boxplot(
            aes(x = x_pos - 0.16, group = level),
            outlier.shape = NA, width = 0.28
        ) +
        geom_jitter(
            aes(x = x_pos + 0.18, colour = ConservationLevel),
            width = 0.12, height = 0,
            size = line_dot_size, alpha = alpha, show.legend = TRUE
        )+

       ylim(0,105) +
        scale_x_continuous(breaks = seq_along(level), labels = level) +
        labs(x=NULL, y="Index incidence (%)\n", color="Conservation level")+
        theme_classic(base_size = base_size)+
        theme(
            legend.key = element_rect(fill = "transparent", colour = "transparent"),
            legend.position = 'bottom',
            plot.margin = unit(c(1, 1, 1, 1), "lines"),
            axis.ticks.x = element_blank(),
            axis.text.x = element_text(vjust = 0.5, hjust=0.5)
        ) +
        scale_colour_manual('Conservation Level',
                            breaks = C_level,
                            values = c("Completely conserved (CC)"="black",
                                       "Highly conserved (HC)"="#0057d1",
                                       "Mixed variable (MV)"="#02d57f",
                                       "Highly diverse (HD)"="#8722ff",
                                       "Extremely diverse (ED)"="#ff617d"),
                            drop = FALSE) +
        coord_cartesian(clip = "off")+ #allow ggtext outside of the plot
        ggtitle(unique(data$host)) +
        theme(plot.title = element_text(margin=margin(b = 50, unit = "pt"))) +
        guides(color = guide_legend(override.aes = list(size = 2), nrow=2))+
        geom_richtext(data = protein_labels,
                      aes(x=x_pos,label = label, y=c(rep(105,nProtein)),
                          label.size=0, label.color="transparent"),
                      position = position_dodge(width=0.1),
                      size=label_size, color="black", hjust=0, angle=90)

}

#' Metadata Extraction from NCBI/GISAID (EpiFlu/EpiCoV/EpiPox/EpiArbo) FASTA file
#'
#' This function retrieves metadata (ID, region, date) from the input FASTA file, with the source of, either 
#' NCBI (with default FASTA header) or GISAID (with default FASTA header). The function will return a dataframe
#' that has three columns consisting ID, collected region and collected date. Records that do not have region or date
#' information will be excluded from the output dataframe.
#'
#' @param file_path path of fasta file
#' @param source the source of fasta file, either "NCBI" or "GISAID"
#' @return  A dataframe that has three columns consisting ID, collected region and collected date
#' @examples filepath <- system.file('extdata','GISAID_EpiCoV.faa', package = 'vDiveR')
#' @examples meta_gisaid <- metadata_extraction(filepath, 'GISAID')
#' @export
metadata_extraction <- function(file_path, source){
    if (tolower(source) == 'ncbi') {
        meta <- extract_from_NCBI(file_path)
    } else if (tolower(source) == 'gisaid') {
        meta <- extract_from_GISAID(file_path)
    } else {
        stop("Invalid source specified. Please use 'NCBI' or 'GISAID'.")
    }
    return(meta)
}

#' Extract metadata via fasta file from GISAID
#'
#' This function get the metadata from each header of GISAID fasta file
#' @param file_path path of fasta file
#' @importFrom stringr str_extract
extract_from_GISAID <- function(file_path){
    lines <- readLines(file_path, warn = FALSE)
    headers <- lines[grepl('^>', lines)]
    
    #================= Extract metadata from headers =====================#
    #e.g: >Spike|hCoV-19/Wuhan/WIV04/2019|2019-12-30|EPI_ISL_402124|
    # id: EPI_ISL_402124
    # region: Wuhan
    # date: 2019-12-30

    ids <- str_extract(headers, "EPI[^|]*")
    regions <- sapply(strsplit(headers, "/"), function(x) if(length(x) > 1) x[2] else NA)
    dates <- str_extract(headers, "\\d{4}-\\d{2}-\\d{2}")
    
    return(data.frame(ID = ids, region = regions, date = dates, stringsAsFactors = FALSE))
}

#' Extract metadata via fasta file from ncbi
#'
#' This function get the metadata from each head of fasta file
#' @param file_path path of fasta file
#' @importFrom rentrez entrez_search entrez_fetch
extract_from_NCBI <- function(file_path){
    #================= Extract metadata from headers =====================#
    #e.g: >QQL10871.1 Rh230 [Rhesus cytomegalovirus strain 68-1_FL]
    DATE_FORMATS <- c('%Y-%m-%d', '%d-%B-%Y', '%m/%d/%Y', '%B %d, %Y', '%d-%m-%Y')
    
    lines <- readLines(file_path, warn=FALSE) # read lines from fasta file into vector
    # extract id from header
    headers <- lines[grepl('^>', lines)]
    headers <- substring(headers, 2)
    IDs <- sapply(strsplit(headers, ' '), function(x) x[1])
    
    
    regions <- character()
    dates <- character()
    drop_sample_not_found <- character()
    drop_sample_no_date_or_region <- character()
    drop_sample_date_format_issue <- character()
    keep_sample <- character()

    # regions <- c(); dates <- c(); drop_sample <- c(); keep_sample <- c()
    for(ID in IDs){
        if(startsWith(ID, 'pdb')){
            drop_sample_not_found <- c(drop_sample_not_found, ID)
            next
        }

        # keep_sample <- c(keep_sample, ID)
        search_result <- rentrez::entrez_search(db = "protein", term = ID, retmax = 1)
        if(search_result$count == 0){
            drop_sample_not_found <- c(drop_sample_not_found, ID)
            next
        }

        uid <- search_result$ids[1] #get unique identifier
        # fetch genbank entry
        genbank_entry <- rentrez::entrez_fetch(db = "protein", id = uid, rettype = "gb", retmode = "text")
        entry_info <- tryCatch(strsplit(genbank_entry, '\n')[[1]], error = function(e) "")
        
        # get region and date index
        region_index <- grep('/country=',  entry_info)
        date_index <- grep('/collection_date=',  entry_info)

        if (length(region_index) == 0){region_index <- grep('/geo_loc_name=',  entry_info)}
        if(length(region_index) == 0 | length(date_index) == 0){
            drop_sample_no_date_or_region <- c(drop_sample_no_date_or_region, ID)
            next
        }

        # extract region and date from the identified line
        region_info <- entry_info[region_index]
        date_info <- entry_info[date_index]

        region <- tryCatch(strsplit(region_info, '\\"')[[1]][2], error = function(e) "NA")
        if(grepl(':', region)){region <- strsplit(region,':')[[1]][1]}
        date <- tryCatch(strsplit(date_info, '\\"')[[1]][2], error = function(e) "NA")
        
        regions <- c(regions, region)
        dates <- c(dates, date)
        keep_sample <- c(keep_sample, ID)
    }

    if (length(regions) == 0 | length(dates) == 0) {
        stop("No valid records found in the input file.")
    }  

    if (length(drop_sample_not_found) > 0) {
        drop_sample_not_found <- unique(drop_sample_not_found)
        warning_info <- paste(c('\nExcluded records:\n',drop_sample_not_found, '\nID not found in the database.'), collapse = " ")
        warning(warning_info)
    }
    if (length(drop_sample_no_date_or_region) > 0) {
        drop_sample_no_date_or_region <- unique(drop_sample_no_date_or_region)
        warning_info <- paste(c('\nExcluded records:\n',drop_sample_no_date_or_region, '\nDate and/or region information is not available.'), collapse = " ")
        warning(warning_info)
    }    

    result_df <- data.frame('ID' = keep_sample, 'region' = regions, 'date' = dates, stringsAsFactors = FALSE)
    result_df$date <- sapply(result_df$date, function(d) { # format and validate dates
        final_date <- NA

        for(fmt in DATE_FORMATS) { # attempt to parse the date with each format
            parsed_date <- as.Date(d, format = fmt)
            if (!is.na(parsed_date)) { # if successful, convert to the standard format '%Y-%m-%d'
                final_date <- as.character(parsed_date)
                break
            }
        }
           
        if (is.na(final_date)) { # if no format matched, add to drop_sample_date_format_issue
            drop_sample_date_format_issue <- c(drop_sample_date_format_issue, result_df$ID[result_df$date == d])
        }
  
        return(final_date)
    })

    if (length(drop_sample_date_format_issue) > 0) {
        drop_sample_date_format_issue <- unique(drop_sample_date_format_issue)
        warning_info <- paste(c('\nExcluded records:\n',drop_sample_date_format_issue, '\nDate in the entry is not in these formats: .', DATE_FORMATS, '\n.'), collapse = " ")
        warning(warning_info)
    }

    all_drops <- c(drop_sample_not_found, drop_sample_no_date_or_region, drop_sample_date_format_issue)
    result_df <- result_df[! result_df$ID %in% all_drops, ] # filter the result_df to exclude IDs in all_drops
    
    return(result_df)
}

#' Dynamics of Diversity Motifs (Proteome) Plot
#'
#' This function compactly display the dynamics of diversity motifs (index and its variants: major, minor and unique)
#' in the form of dot plot as well as violin plot for all the provided proteins at proteome level.
#'
#' @param df DiMA JSON converted csv file data
#' @param host number of host (1/2)
#' @param line_dot_size size of dot in plot
#' @param base_size word size in plot
#' @param alpha any number from 0 (transparent) to 1 (opaque)
#' @param bw smoothing bandwidth of violin plot (default: nrd0)
#' @param adjust adjust the width of violin plot (default: 1)
#' @return A plot
#' @examples plot_dynamics_proteome(proteins_1host)
#' @importFrom cowplot plot_grid
#' @export
plot_dynamics_proteome <- function(df,
                                   host=1,
                                   line_dot_size=2,
                                   base_size=10,
                                   alpha=1/3,
                                   bw = "nrd0",
                                   adjust = 1){
    #single host
    if (host == 1){
        generate_plots(data=df, base_size= base_size, alpha=alpha, line_dot_size=line_dot_size, bw = bw, adjust = adjust)
    }else{ #multihost
        #split the data into multiple subsets (if multiple hosts detected)
        data_list<-split(df,df$host)
        multihost_plots <- lapply(data_list, function(df) {
            generate_plots(df, line_dot_size = line_dot_size, base_size = base_size, 
                    alpha = alpha, bw = bw, adjust = adjust, host = host)
        })
        
        #create spacing between multihost plots
        theme = theme(plot.margin = unit(c(0,0.1,0,0), "cm"))
        plot_grid(plotlist = lapply(multihost_plots, '+', theme),
                  ncol = length(unique(df$host)))
    }

}

#' @importFrom ggplot2 element_blank facet_wrap scale_colour_manual
#' @importFrom ggplot2 guides guide_legend scale_color_manual ggtitle element_text geom_violin geom_boxplot ylim scale_color_grey margin scale_fill_manual coord_cartesian
#' @importFrom ggpubr annotate_figure ggarrange text_grob
#' @importFrom patchwork plot_layout wrap_plots
generate_plots<-function(data,
                        line_dot_size=2,
                        base_size=10,
                        host = 1,
                        alpha=1/3,
                        bw = "nrd0",
                        adjust = 1){

    Total_Variants <- Incidence <- Group <- x <- NULL
    df<-data.frame()
    group_names<-c("Index",
                   "Major", "Minor", "Unique",
                   "Total variants","Distinct variants")

    #transpose the data format
    for (i in 7:12){
        tmp<-data.frame(proteinName=data[1],
                        position=data[2],
                        incidence=data[i],
                        total_variants=data[11],
                        Group=group_names[i-6],
                        Multiindex=data[13])
        names(tmp)[3]<-"Incidence"
        names(tmp)[4]<-"Total_Variants"
        df<-rbind(df,tmp)
    }

    df_violin<-df
    minor<-rbind(df[df$Group == "Index",],df[df$Group == "Total variants",])
    uniq<-rbind(df[df$Group == "Index",],df[df$Group == "Total variants",])
    minor$motif<- "Minor"
    uniq$motif<-"Unique"

    df<-df%>%mutate(motif = case_when(
        df$Group == "Index" ~ "Major",
        df$Group == "Total variants" ~ "Major",
        df$Group == "Major" ~ "Major",
        df$Group == "Minor"  ~ "Minor",
        df$Group == "Unique" ~ "Unique",
        df$Group == "Distinct variants" ~ "Distinct variants"
    ))
    df<- rbind(df,minor,uniq)
    df$motif<-factor(df$motif,levels = c("Major","Minor","Unique","Distinct variants"))

    # Define common theme components
    common_left <- theme(
        axis.text.y  = element_text(size = base_size),
        axis.ticks.y = element_line(),
        plot.margin  = margin(t = 5, r = 5, b = 5, l = 5)
    )
    
    theme_clean <- theme_classic(base_size = base_size) +
        theme(
            panel.border = element_rect(colour = "black", fill=NA, linewidth=1),
            plot.title   = element_text(size = base_size, face = "bold", hjust = 0.5),
            axis.title.x = element_blank(),
            axis.title.y = element_blank(),
            axis.text.x  = element_blank(),
            axis.ticks.x = element_blank()
        ) + common_left

    #plotting scatter plot
    proteins_point_plot<-ggplot()+
        geom_point(df,mapping=aes(x=Total_Variants,y=Incidence,color=Group),alpha=alpha,size= line_dot_size)+
        geom_point(df,mapping = aes(x =Total_Variants,y=Incidence),
                   col=ifelse(df$multiIndex== TRUE & df$Group== "Index", 'red', ifelse(df$multiIndex== FALSE, 'white', 'white')), 
                   alpha=ifelse(df$multiIndex ==TRUE & df$Group== "Index", 1, ifelse(df$multiIndex== TRUE, 0,0)),
                   pch=1,size=3,stroke=1.05)+ #multiIndex
        scale_x_continuous(limits = c(0, 100), breaks = seq(0, 100, 20))+
        scale_y_continuous(limits = c(0, 100), breaks = seq(0, 100, 20))+
        theme_classic(base_size = base_size) +
        theme(
            panel.border = element_rect(colour = "black", fill=NA, linewidth=1),
            plot.title   = element_text(size = base_size, face = "bold"),
            axis.title.x = element_text(size = base_size, face = "bold"),
            axis.title.y = element_text(size = base_size, face = "bold"),
            strip.text.x = element_blank(),
            legend.position="bottom"
        ) + common_left +
        labs(y= "Incidence (%)", x="\nTotal variants (%)")+
        facet_wrap(~ motif,ncol = 1)+
        guides(colour = guide_legend(override.aes = list(alpha = 1,size=2), nrow = host, byrow=T, keywidth = 1, keyheight = .1 ))+
        scale_colour_manual('',breaks=c("Index","Total variants","Major","Minor","Unique","Distinct variants"),
                            values = c("Index"="black", "Total variants"="#f7238a","MultiIndex"="red",
                                       "Major"="#37AFAF" , "Minor"="#42aaff","Unique"="#af10f1", "Distinct variants"="#c2c7cb"))
    #host label
    if("host" %in% colnames(data)){
        proteins_point_plot<-proteins_point_plot+ggtitle(unique(data$host))
    }

    # Create individual violin plots
    index<-df_violin[df_violin$Group %in% "Index",]
    total_var<-df_violin[df_violin$Group %in% "Total variants",]
    distinct_var<-df_violin[df_violin$Group %in% "Distinct variants",]
    major_var<-df_violin[df_violin$Group %in% "Major",]
    minor_var<-df_violin[df_violin$Group %in% "Minor",]
    unique_var<-df_violin[df_violin$Group %in% "Unique",]

    p_index <- ggplot(index, aes(x = "", y = Incidence)) +
        geom_violin(fill = "black", color = "black", bw = bw, adjust = adjust) +
        geom_boxplot(width=0.08, alpha=0.20, fill="white", outlier.shape=NA, color="white") +
        coord_cartesian(ylim = c(0, 100)) +
        ggtitle("Index") + theme_clean

    p_total <- ggplot(index, aes(x = "", y = Total_Variants)) +
        geom_violin(fill = "#f7238a", color = "#f7238a", bw = bw, adjust = adjust) +
        geom_boxplot(width = 0.1, outlier.shape = NA, fill = NA) +
        coord_cartesian(ylim = c(0, 100)) +
        ggtitle("Total variants") + theme_clean

    p_distinct <- ggplot(distinct_var, aes(x = "", y = Incidence)) +
        geom_violin(fill = "#c2c7cb", color = "#c2c7cb", bw = bw, adjust = adjust) +
        geom_boxplot(width = 0.1, outlier.shape = NA, fill = NA) +
        coord_cartesian(ylim = c(0, 100)) +
        ggtitle("Distinct variants") + theme_clean

    p_major <- ggplot(major_var, aes(x = "", y = Incidence)) +
        geom_violin(fill = "#37AFAF", color = "#37AFAF", bw = bw, adjust = adjust) +
        geom_boxplot(width = 0.1, outlier.shape = NA, fill = NA) +
        coord_cartesian(ylim = c(0, 50)) +
        ggtitle("Major variants") + theme_clean

    p_minor <- ggplot(minor_var, aes(x = "", y = Incidence)) +
        geom_violin(fill = "#42aaff", color = "#42aaff", bw = bw, adjust = adjust) +
        geom_boxplot(width = 0.1, outlier.shape = NA, fill = NA) +
        coord_cartesian(ylim = c(0, 50)) +
        ggtitle("Minor variants") + theme_clean

    p_unique <- ggplot(unique_var, aes(x = "", y = Incidence)) +
        geom_violin(fill = "#af10f1", color = "#af10f1", bw = bw, adjust = adjust) +
        geom_boxplot(width = 0.1, outlier.shape = NA, fill = NA) +
        coord_cartesian(ylim = c(0, 50)) +
        ggtitle("Unique variants") + theme_clean

    # 2 × 3 layout: Index | Total | Distinct / Major | Minor | Unique
    violin_panel <- (p_index | p_total | p_distinct) / (p_major | p_minor | p_unique)

    # Combine scatter plot and violin panel
    combined_plot <- proteins_point_plot / violin_panel +
        plot_layout(heights = c(1.6, 1))

    return(combined_plot)
}

#' Dynamics of Diversity Motifs (Protein) Plot
#'
#' This function compactly display the dynamics of diversity motifs (index and its variants: major, minor and unique)
#' in the form of dot plot(s) as well as violin plots for all the provided individual protein(s).
#'
#' @param df DiMA JSON converted csv file data
#' @param host number of host (1/2)
#' @param protein_order order of proteins displayed in plot
#' @param base_size base font size in plot
#' @param alpha any number from 0 (transparent) to 1 (opaque)
#' @param line_dot_size dot size in scatter plot
#' @param bw smoothing bandwidth of violin plot (default: ucv)
#' @param adjust adjust the width of violin plot (default: 1)
#' @return A plot
#' @examples plot_dynamics_protein(proteins_1host)
#' @importFrom cowplot plot_grid ggdraw draw_label draw_plot
#' @importFrom ggplot2 ggplot aes geom_point geom_violin geom_boxplot scale_x_continuous scale_y_continuous
#' @importFrom ggplot2 labs theme_classic theme element_rect element_text element_blank guides guide_legend
#' @importFrom ggplot2 scale_colour_manual scale_fill_manual facet_grid ggtitle coord_cartesian vars xlab ylab
#' @export
plot_dynamics_protein<-function(df, 
                                host=1, 
                                protein_order=NULL, 
                                base_size=11, 
                                alpha = 1/3, 
                                line_dot_size = 3,
                                bw = "ucv",
                                adjust = 1){
    #single host
    if (host == 1){
        generate_protein_plots(data=df, protein_order=protein_order, base_size=base_size,alpha=alpha,  line_dot_size=line_dot_size, bw = bw, adjust = adjust)
    }else{ #multihost
        #split the data into multiple subsets (if multiple hosts detected)
        data_list<-split(df,df$host)
        multihost_plots <- lapply(data_list, function(df) {      
          generate_protein_plots(df, protein_order = protein_order, base_size = base_size, alpha = alpha,
                  line_dot_size = line_dot_size, bw = bw, adjust = adjust, host=host)
        })

        plot_grid(plotlist = multihost_plots,
              ncol = length(unique(df$host)))
    }
}

#' @importFrom ggpubr ggarrange
#' @importFrom cowplot plot_grid ggdraw draw_label draw_plot
generate_protein_plots<-function(data, protein_order=NULL,alpha=1/3, line_dot_size=3, base_size=11, host=1, bw = "ucv", adjust = 1){
    Total_Variants <- Incidence <- Group <- x <- proteinName <- entropy <- NULL

    scatter_plot_data<-data.frame()
    group_names<-c("Index",
                   "Major","Minor",
                   "Unique",
                   "Total variants","Distinct variants")

    for (i in 7:12){
        tmp<-data.frame(proteinName=data[1],position=data[2],incidence=data[i],total_variants=data[11],Group=group_names[i-6],Multiindex=data[13])

        names(tmp)[3]<-"Incidence"
        names(tmp)[4]<-"Total_Variants"
        scatter_plot_data<-rbind(scatter_plot_data,tmp)
    }

    scatter_plot_data$proteinName <- toupper(scatter_plot_data$proteinName)
    if (!is.null(protein_order) && protein_order != ""){
        #order the proteins based on user input
        protein_order <- toupper(trimws(protein_order))
        level<-strsplit(protein_order, ',')[[1]]
        level <- sapply(level, function(x) toupper(trimws(x)))
        #set protein order as factor
        
        scatter_plot_data$proteinName<-factor(scatter_plot_data$proteinName, levels=level)
        scatter_plot_data$size_f = factor(scatter_plot_data$proteinName,levels = level)
    }
    scatter_plot_data$Group<-factor(scatter_plot_data$Group, levels=c("Index","Total variants", "Major", "Minor", "Unique", "Distinct variants"))
    violin_plot_data<-scatter_plot_data

    #plot plot 4
    scatter_plots<-ggplot()+geom_point(scatter_plot_data,mapping=aes(x=Total_Variants,y=Incidence,color=Group),alpha=alpha,size=line_dot_size)+
        scale_x_continuous(limits = c(0, 100), breaks = seq(0, 100, 20))+
        scale_y_continuous(limits = c(0, 100), breaks = seq(0, 100, 20))+
        labs(y = NULL,x= NULL)+
        theme_classic(base_size = base_size)+
        theme(
            legend.background = element_rect(fill = "transparent"),
            panel.border = element_rect(colour = "black", fill=NA, linewidth=1),
            plot.title   = element_text(size = base_size, face = "bold"),
            axis.title.x = element_text(size = base_size, face = "bold"),
            axis.title.y = element_text(size = base_size, face = "bold"),
            strip.text.x = element_text(size = base_size-1, face = "bold"),
            legend.position = "bottom"
        )+ guides(colour = guide_legend(override.aes = list(alpha = 1,size=2),keywidth = 1,keyheight = 0.1,nrow=1, byrow=TRUE))+
        scale_colour_manual('',values = c("Index"="black","Total variants"="#f7238a", "Major"="#37AFAF","Minor"="#42aaff","Unique"="#af10f1","Distinct variants"="#c2c7cb" ))
    scatter_plots<-scatter_plots+facet_grid(cols=vars(scatter_plot_data$proteinName))

    #host label
    if("host" %in% colnames(data)){
        scatter_plots<-scatter_plots+ggtitle(unique(data$host))+
            theme(plot.title = element_text(hjust = 0.5))
    }
    
    # Add y-label to scatter plot
    scatter_with_ylabel <- ggdraw() +
        draw_label("Incidence (%)",
                   x = 0.02, y = 0.5,
                   angle = 90,
                   fontface = "bold", size = base_size) +
        draw_plot(scatter_plots, x = 0.06, y = 0, width = 0.94, height = 1)
    
    if (length(unique(data$proteinName)) <=10){
      # Define violin plot theme
      theme_violin <- theme_classic(base_size = base_size) +
          theme(
              panel.border = element_rect(colour = "black", fill = NA, linewidth = 1),
              plot.title   = element_text(size = base_size, face = "bold", hjust = 0.5),
              axis.title.x = element_blank(),
              axis.title.y = element_blank()
          )
      
      #prepare the data for each subplot of violin_plots
      index<-violin_plot_data[violin_plot_data$Group %in% c("Index"),]
      major<-violin_plot_data[violin_plot_data$Group %in% c("Major"),]
      minor<-violin_plot_data[violin_plot_data$Group %in% c("Minor"),]
      unique<-violin_plot_data[violin_plot_data$Group %in% c("Unique"),]
      nonatypes<-violin_plot_data[violin_plot_data$Group %in% c("Distinct variants"),]
      variants_max_yaxis<-ceiling((max(as.numeric(major$Incidence),as.numeric(minor$Incidence),as.numeric(unique$Incidence))/10))*10
  
      #plot 5
      violin_plot_index<-ggplot(index, aes(x=proteinName, y=Incidence))+
          geom_violin(fill="black",trim = TRUE, color="black",alpha=0.9, adjust=adjust, bw=bw)+
          geom_boxplot(outlier.shape = NA,width=0.08, color="white",alpha=0.20,fill="white")+
          coord_cartesian(ylim = c(0, 100))+
          ggtitle("Index k-mer (%)")+
          theme_violin
  
      violin_plot_tv<-ggplot(index, aes(x=proteinName, y=Total_Variants))+
          geom_violin(fill="#f7238a",trim = TRUE, color="#f7238a",alpha=0.9, adjust=adjust, bw=bw)+
          geom_boxplot(outlier.shape = NA,width=0.08, color="black",alpha=0.20,fill=NA)+
          coord_cartesian(ylim = c(0, 100))+
          ggtitle("Total variants (%)")+
          theme_violin
  
      violin_plot_major<-ggplot(major, aes(x=proteinName, y=Incidence)) +
          geom_violin(fill="#37AFAF",trim = TRUE, color="#37AFAF", alpha=0.9, adjust=adjust, bw=bw)+
          geom_boxplot(outlier.shape = NA,width=0.08, color="black", alpha=0.20,fill=NA)+
          coord_cartesian(ylim = c(0, variants_max_yaxis))+
          ggtitle("Major variants (%)")+
          theme_violin
  
      violin_plot_minor<-ggplot(minor, aes(x=proteinName, y=Incidence))+
          geom_violin(fill="#42aaff",trim = TRUE,color="#42aaff", alpha=0.9, adjust=adjust, bw=bw)+
          geom_boxplot(outlier.shape = NA,width=0.08, color="black", alpha=0.20,fill=NA)+
          coord_cartesian(ylim = c(0, variants_max_yaxis))+
          ggtitle("Minor variants (%)")+
          theme_violin
  
      violin_plot_unique<-ggplot(unique, aes(x=proteinName, y=Incidence)) +
          geom_violin(fill="#af10f1",trim = TRUE, color="#af10f1", alpha=0.9, adjust=adjust, bw=bw)+
          geom_boxplot(outlier.shape = NA,width=0.08, color="black", alpha=0.20,fill=NA)+
          coord_cartesian(ylim = c(0, variants_max_yaxis))+
          ggtitle("Unique variants (%)")+
          theme_violin
  
      violin_plot_nonatypes<-ggplot(nonatypes, aes(x=proteinName, y=Incidence)) +
          geom_violin(fill="#c2c7cb",trim = TRUE, color="#c2c7cb", alpha=0.9, adjust=adjust, bw=bw)+
          geom_boxplot(outlier.shape = NA,width=0.08, color="black", alpha=0.20,fill=NA) +
          coord_cartesian(ylim = c(0, 100))+
          ggtitle("Distinct variants (%)")+
          theme_violin
      
      violin_plots<-ggarrange(violin_plot_index,violin_plot_tv,violin_plot_nonatypes,violin_plot_major,violin_plot_minor,violin_plot_unique,ncol=3,nrow=2)
      
      # Add y-label to violin panel
      violin_with_ylabel <- ggdraw() +
          draw_label("Incidence (%)",
                     x = 0.02, y = 0.5,
                     angle = 90,
                     fontface = "bold", size = base_size) +
          draw_plot(violin_plots, x = 0.06, y = 0, width = 0.94, height = 1)
      
    } else {
      violin_plot_data$Group[violin_plot_data$Group == "Index"] <- "Index k-mer" 
      violin_plot_data$Group[violin_plot_data$Group == "Major"] <- "Major variant" 
      violin_plot_data$Group[violin_plot_data$Group == "Minor"] <- "Minor variants" 
      violin_plot_data$Group[violin_plot_data$Group == "Unique"] <- "Unique variants" 
      
      violin_plot_data$Group<-factor(violin_plot_data$Group, levels=c("Index k-mer","Total variants", "Distinct variants", "Major variant", "Minor variants", "Unique variants"))
      variants<-subset(violin_plot_data, Group=="Major variant" | Group=="Minor variants" | Group=="Unique variants")
      max_ylim<-ceiling((max(variants$Incidence)/10))*10
      
      breaks_fun <- function(x) {
        if (max(x)<= max_ylim){
          seq(0,max_ylim,10)
        }else{
          seq(0,100,20)
        }
      }
      
      limits_fun <- function(x) {
        if (max(x)<= max_ylim){
          c(0,max_ylim)
        }else{
          c(0,100)
        }
      }
      
      violin_plots<-ggplot()+
        geom_violin(data=violin_plot_data,aes(x=proteinName,y=Incidence, fill=Group, color=Group), trim=TRUE, adjust=adjust, bw=bw)+
        theme_classic(base_size = base_size)+xlab("Protein")+ylab("Incidence (%)\n")+
        theme(panel.border = element_rect(colour = "black", fill=NA, linewidth=1),
              legend.position="none")+
        scale_y_continuous(limits = limits_fun,breaks = breaks_fun)+
        facet_grid(rows = vars(Group),switch="y",scales = 'free')+
        scale_colour_manual('',values = c("Index k-mer"="black","Total variants"="#f7238a", "Major variant"="#37AFAF","Minor variants"="#42aaff","Unique variants"="#af10f1","Nonatypes"="#c2c7cb" ))+
        scale_fill_manual('',values = c("Index k-mer"="black","Total variants"="#f7238a", "Major variant"="#37AFAF","Minor variants"="#42aaff","Unique variants"="#af10f1","Nonatypes"="#c2c7cb" ))
      
      violin_with_ylabel <- violin_plots
    }
    
    plot_grid(scatter_with_ylabel, violin_with_ylabel, ncol = 1, rel_heights = c(1.6, 1))
}

#' DiMA (v5.0.9) JSON converted-CSV Output Sample 1
#'
#' A dummy dataset with two proteins (A and B) from one host, human
#'
#'
#' @format A data frame with 806 rows and 17 variables:
#' \describe{
#'   \item{proteinName}{name of the protein}
#'   \item{position}{starting position of the aligned, overlapping k-mer window}
#'   \item{count}{number of k-mer sequences at the given position}
#'   \item{lowSupport}{k-mer position with sequences lesser than the minimum support threshold (TRUE) are considered of low support, in terms of sample size}
#'   \item{entropy}{level of variability at the k-mer position, with zero representing completely conserved}
#'   \item{indexSequence}{the predominant sequence (index motif) at the given k-mer position}
#'   \item{index.incidence}{the fraction (in percentage) of the index sequences at the k-mer position}
#'   \item{major.incidence}{the fraction (in percentage) of the major sequence (the predominant variant to the index) at the k-mer position}
#'   \item{minor.incidence}{the fraction (in percentage) of minor sequences (of frequency lesser than the major variant, but not singletons) at the k-mer position}
#'   \item{unique.incidence}{the fraction (in percentage) of unique sequences (singletons, observed only once) at the k-mer position}
#'   \item{totalVariants.incidence}{the fraction (in percentage) of sequences at the k-mer position that are variants to the index (includes: major, minor and unique variants)}
#'   \item{distinctVariant.incidence}{incidence of the distinct k-mer peptides at the k-mer position}
#'   \item{multiIndex}{presence of more than one index sequence of equal incidence}
#'   \item{host}{species name of the organism host to the virus}
#'   \item{highestEntropy.position}{k-mer position that has the highest entropy value}
#'   \item{highestEntropy}{highest entropy values observed in the studied protein}
#'   \item{averageEntropy}{average entropy values across all the k-mer positions}
#' }
"proteins_1host"

#' DiMA (v5.0.9) JSON converted-CSV Output Sample 2
#'
#' A dummy dataset with 1 protein (Core) from two hosts, human and bat
#'
#'
#' @format A data frame with 200 rows and 17 variables:
#' \describe{
#'   \item{proteinName}{name of the protein}
#'   \item{position}{starting position of the aligned, overlapping k-mer window}
#'   \item{count}{number of k-mer sequences at the given position}
#'   \item{lowSupport}{k-mer position with sequences lesser than the minimum support threshold (TRUE) are considered of low support, in terms of sample size}
#'   \item{entropy}{level of variability at the k-mer position, with zero representing completely conserved}
#'   \item{indexSequence}{the predominant sequence (index motif) at the given k-mer position}
#'   \item{index.incidence}{the fraction (in percentage) of the index sequences at the k-mer position}
#'   \item{major.incidence}{the fraction (in percentage) of the major sequence (the predominant variant to the index) at the k-mer position}
#'   \item{minor.incidence}{the fraction (in percentage) of minor sequences (of frequency lesser than the major variant, but not singletons) at the k-mer position}
#'   \item{unique.incidence}{the fraction (in percentage) of unique sequences (singletons, observed only once) at the k-mer position}
#'   \item{totalVariants.incidence}{the fraction (in percentage) of sequences at the k-mer position that are variants to the index (includes: major, minor and unique variants)}
#'   \item{distinctVariant.incidence}{incidence of the distinct k-mer peptides at the k-mer position}
#'   \item{multiIndex}{presence of more than one index sequence of equal incidence}
#'   \item{host}{species name of the organism host to the virus}
#'   \item{highestEntropy.position}{k-mer position that has the highest entropy value}
#'   \item{highestEntropy}{highest entropy values observed in the studied protein}
#'   \item{averageEntropy}{average entropy values across all the k-mer positions}
#' }
"protein_2hosts"

#' DiMA (v5.0.9) JSON Output File
#'
#' A sample DiMA JSON Output File which acts as the input for JSON2CSV()
#'
#'
#' @format A Diversity Motif Analyzer (DiMA) tool JSON file
"JSON_sample"

#' Metadata Input Sample
#'
#' A dummy dataset that acts as an input for plot_world_map() and plot_time()
#'
#'
#' @format A data frame with 1000 rows and 3 variables:
#' \describe{
#'   \item{ID}{unique identifier of the sequence}
#'   \item{region}{geographical region of the sequence collection}
#'   \item{date}{collection date of the sequence}
#' }
"metadata"

#' Time Distribution of Sequences Plot
#'
#' This function plots the time distribution of provided sequences in the form of bar plot with 'Month' as x-axis and
#' 'Number of Sequences' as y-axis. Aside from the plot, this function also returns a dataframe with 2 columns: 'Date' and 'Number of sequences'.
#' The input dataframe of this function is obtainable from metadata_extraction(), with NCBI Protein / GISAID (EpiFlu/EpiCoV/EpiPox/EpiArbo) FASTA file as input.
#'
#' @param metadata a dataframe with 3 columns, 'ID', 'region', and 'date'
#' @param base_size word size in plot
#' @param date_format date format of the input dataframe
#' @param date_break date break for the scale_x_date
#' @param scale plot counts or log scale the data
#'
#' @return A single plot or a list with 2 elements (a plot followed by a dataframe, default)
#' @examples time_plot <- plot_time(metadata, date_format="%d/%m/%Y")$plot
#' @examples time_df <- plot_time(metadata, date_format="%d/%m/%Y")$df
#' @importFrom ggplot2 scale_x_date geom_bar scale_y_log10
#' @importFrom scales date_format trans_breaks trans_format math_format
#' @importFrom stringr str_to_title
#' @importFrom magrittr %>%
#' @importFrom dplyr filter group_by summarize
#' @export
plot_time <- function(metadata, date_format = "%Y-%m-%d", base_size=8,
                      date_break = "2 month", scale = "count"){
    Month <- .x <- Date <- NULL
    colnames(metadata) <- stringr::str_to_title(colnames(metadata))
    
    dates_3rows<-paste(metadata$Date[1:3], collapse = ", ")
    metadata <- metadata %>%
        filter(!is.na(as.Date(metadata$Date, format=date_format, na.rm = TRUE)))

    if (nrow(metadata) < 1){
        error_msg <- paste("No records remain after filtering out those not in date format of", date_format, ". Have a peek on some 'Date' values:", dates_3rows)
        return(list(plot = NULL, df = error_msg))
    }

    metadata$Month <- as.Date(cut(as.Date(metadata$Date, format = date_format),
                                  breaks = "month"))

    p <- ggplot(data = metadata, aes(x = Month)) + 
        geom_bar() +
        ylab('Number of Sequences') +
        scale_x_date(date_breaks = date_break, labels = scales::date_format("%b %Y"))  +
        theme_classic(base_size = base_size) +
        theme(axis.text.x = element_text(angle = 90, vjust = .5))

    if (scale == 'log') {
        p <- p + scale_y_log10(breaks = trans_breaks("log10", function(x) 10^x),
                                 labels = trans_format("log10", scales::math_format(10^.x)))
    }

    final_df <- metadata %>%
        dplyr::group_by(Date) %>%
        dplyr::summarize("Number of Sequences" = n()) %>%
        as.data.frame()

    return(list(plot=p,df=final_df))
}

#' k-mer sequences concatenation
#'
#' This function concatenates completely ( index incidence = 100\% ) only or both 
#' both the completely and highly ( 90\% <= index incidence < 100\% ) conserved 
#' k-mer positions that overlapped at least one k-mer position or are
#' adjacent to each other and generate the CCS/HCS sequence in either CSV or FASTA format
#'
#' @param data DiMA JSON converted csv file data
#' @param conservation_level CCS (completely conserved) / HCS (highly conserved)
#' @param kmer size of the k-mer window
#' @param threshold_pct manually set threshold of index.incidence for HCS 
#' @return A list wit csv and fasta dataframes
#' @examples csv<-concat_conserved_kmer(proteins_1host)$csv
#' @examples csv_2hosts<-concat_conserved_kmer(protein_2hosts, conservation_level = "CCS")$csv
#' @examples fasta <- concat_conserved_kmer(protein_2hosts, conservation_level = "HCS")$fasta
#' @importFrom magrittr %>%
#' @importFrom dplyr filter select summarise group_by slice bind_rows mutate n
#' @importFrom stringr str_sub
#' @importFrom tidyr spread separate
#' @importFrom rlang :=
#' @export
concat_conserved_kmer <- function(data, 
                                  conservation_level = "HCS", 
                                  kmer=9, 
                                  threshold_pct = NULL){
  index.incidence <- proteinName <- indexSequence <- n <- start <- end <- NULL
  # threshold HCS / CCS
  if (is.null(threshold_pct)) {
    threshold <- ifelse(conservation_level == "CCS", 100, 90)
  } else {
    message(sprintf("Manually selected threshold for HCS: %s", threshold_pct))
    threshold <- threshold_pct
  }
  
  
  # filter whole dataset by index.incidence (HCS/CCS)
  df <- data %>%
    dplyr::filter(index.incidence >= threshold)
  
  # # stop if no peptides were found
  # if (nrow(df) == 0) {
  #   # stop("No sequences with a given conservation level were found", call.=FALSE)
  #   message("No sequences with a given conservation level were found")
  #   return()
  # }
  
  # ---- 1. Protein Sequences ----
  
  # remember whole sequence of the protein
  proteins_seq <- data %>%
    dplyr::select(proteinName, indexSequence) %>%
    dplyr::group_by(proteinName) %>%
    dplyr::summarise(seq = paste0(str_sub(indexSequence, 1, 1), collapse = "")) %>%
    tidyr::spread(key = proteinName, value = seq)
  
  # add missing last amino acids
  for (protein in unique(data$proteinName)) {
    proteins_seq[[protein]] <-
      paste0(proteins_seq[[protein]],
             data %>%
               dplyr::filter(proteinName == protein) %>%
               dplyr::select(indexSequence) %>%
               dplyr::slice(n()) %>%
               as.character() %>%
               stringr::str_sub(2))
  }
  
  
  
  
  # ---- 2. Dataset manipulations ----
  
  
  # first split by protein name
  # then separately proceed with each table (each protein independently)
  # then rbind
  
  csv_df <- bind_rows(
    
    lapply(split(df, df$proteinName), function(df_x) {
      
      # remember protein name
      prot_name <- df_x[1, "proteinName", drop = TRUE]
      
      # create table of all indexes falling into peptides (from filtered df)
      # cols: indexes of all amino acids
      # rows: peptides
      # value: TRUE for all amino acids from peptide on their indexes
      # example: row VKRP (from 22 to 25) - cols 22-25 will have TRUE
      index <-
        bind_rows(
          apply(df_x, 1, function(row) {
            start_pos <- as.numeric(row["position"])
            data.frame(matrix(data = TRUE,
                              nrow = 1, ncol = kmer,
                              dimnames = list(row["indexSequence"],
                                              seq(start_pos, start_pos + kmer - 1))))
          })
          
        ) %>%
        # if for any index there is amino acid from conservative peptide - set TRUE
        apply(2, any) %>% names() %>% as.data.frame() %>%
        # get rid of "X" in front of indexes
        tidyr::separate(col = 1, into = c("x", "index"), sep = 1) %>%
        dplyr::mutate(index = as.integer(index)) %>%
        dplyr::select(index)
      
      # set start and end indexes for each consecutive index series
      # calculated as difference between previous and next elements
      # diff = 1 means that sequence still continuing
      # diff > 1 means that there was a gap -> next sequence started
      # example: 4-5-6-10-11-12 // diff = 1-1-4-1-1
      # sequences: 4-6 (ind.1-3), 10-12 (ind.4-6)
      start_ind <- index$index[c(1, which(diff(index$index) > 1) + 1)]
      end_ind <- index$index[c(which(diff(index$index) > 1), length(index$index))]
      
      # format columns for output
      index_df <- data.frame(
        n = seq(length(start_ind)),
        start = start_ind,
        end = end_ind
      ) %>%
        dplyr::mutate(
          !!conservation_level := sprintf("%s_%s_%i", conservation_level, prot_name, n),
          Position = sprintf("%i-%i", start, end),
          Sequence = str_sub(proteins_seq[[prot_name]], start, end)
        ) %>%
        dplyr::select(-c(n, start, end))
      
    })
    
  )
  
  
  if (nrow(csv_df) == 0) {
    cons_lvl <- ifelse(conservation_level == "HCS", 
                       "highly conserved",
                       "completely conserved")
    message_df <- data.frame(
      Warning_message = c(sprintf("No %s sequences were found!", cons_lvl)))
    message(sprintf("No %s sequences were found!", cons_lvl))
    return(list(csv=message_df,
                fasta=message_df))
  }
  
  # create df to store info for fasta file
  fasta_df <- do.call(rbind, lapply(seq(nrow(csv_df)), function(i) {
    csv_df[i, ] %>%
      dplyr::select(-Position) %>%
      dplyr::mutate(!!conservation_level := paste0(">", get(conservation_level))) %>%
      t()
  }))
  
  return(list(csv=csv_df, fasta=fasta_df))
  
  
}

test_that("Test Case 1: 9-mer HCS (single host), with CSV output",{
    HCS_1host<- concat_conserved_kmer(proteins_1host)$csv

    expect_equal(nrow(HCS_1host),7)
    expect_equal(ncol(HCS_1host),3)

    col_names<-colnames(HCS_1host)
    expect_equal(col_names[1],'HCS')
    expect_equal(col_names[2],'Position')
    expect_equal(col_names[3],'Sequence')

    #randomly pick 3 rows to check
    #row 1
    firstRow <- HCS_1host[1,]
    expect_equal(firstRow$HCS, 'HCS_A_1')
    expect_equal(firstRow$Position, '1-18')
    expect_equal(firstRow$Sequence, 'MSTNPKPQRKTKRNTNRR')

    #row 3
    thirdRow <- HCS_1host[3,]
    expect_equal(thirdRow$HCS, 'HCS_A_3')
    expect_equal(thirdRow$Position, '50-67')
    expect_equal(thirdRow$Sequence, 'RKTSERSQPRGRRQPIPK')

    #last row, row 7
    lastRow <- HCS_1host[7,]
    expect_equal(lastRow$HCS, 'HCS_B_2')
    expect_equal(lastRow$Position, '154-166')
    expect_equal(lastRow$Sequence, 'FRAAVCTRGVAKA')
})

test_that("Test Case 2: 9-mer CCS (single host), with CSV output",{
    CCS_1host<- concat_conserved_kmer(proteins_1host, conservation_level = 'CCS')$csv

    expect_equal(nrow(CCS_1host),1)
    expect_equal(ncol(CCS_1host),3)

    col_names<-colnames(CCS_1host)
    expect_equal(col_names[1],'CCS')
    expect_equal(col_names[2],'Position')
    expect_equal(col_names[3],'Sequence')

    #row 1
    firstRow <- CCS_1host[1,]
    expect_equal(firstRow$CCS, 'CCS_A_1')
    expect_equal(firstRow$Position, '1-18')
    expect_equal(firstRow$Sequence, 'MSTNPKPQRKTKRNTNRR')

})

test_that("Test Case 3: 9-mer HCS (single host), with FASTA output",{
    HCS_1host<- concat_conserved_kmer(proteins_1host)$fasta

    expect_equal(nrow(HCS_1host),14)
    expect_equal(ncol(HCS_1host),1)

    #randomly pick 3 FASTA to check
    expect_equal(HCS_1host[1,1], '>HCS_A_1')
    expect_equal(HCS_1host[2,1], 'MSTNPKPQRKTKRNTNRR')

    expect_equal(HCS_1host[5,1], '>HCS_A_3')
    expect_equal(HCS_1host[6,1], 'RKTSERSQPRGRRQPIPK')

    expect_equal(HCS_1host[11,1], '>HCS_B_1')
    expect_equal(HCS_1host[12,1], 'TSLTGRDKN')
})

test_that("Test Case 4: 9-mer CCS (single host), with FASTA output",{
    CCS_1host<- concat_conserved_kmer(proteins_1host, conservation_level = 'CCS')$fasta

    expect_equal(nrow(CCS_1host),2)
    expect_equal(ncol(CCS_1host),1)

    expect_equal(CCS_1host[1,1], '>CCS_A_1')
    expect_equal(CCS_1host[2,1], 'MSTNPKPQRKTKRNTNRR')

})

#' Geographical Distribution of Sequences Plot
#'
#' This function plots a world map and color the affected geographical region(s)
#' from light (lower) to dark (higher), depends on the cumulative number of sequences.
#' Aside from the plot, this function also returns a dataframe with 2 columns: 'Region' and 'Number of Sequences'.
#' The input dataframe of this function is obtainable from metadata_extraction(), with NCBI
#' Protein / GISAID (EpiFlu/EpiCoV/EpiPox/EpiArbo) FASTA file as input.

#' @param metadata a dataframe with 3 columns, 'ID', 'region', and 'date'
#' @param base_size word size in plot
#'
#' @return A list with 2 elements (a plot followed by a dataframe)
#' @examples
#' \dontrun{
#' geographical_plot <- plot_world_map(metadata)$plot
#' geographical_df <- plot_world_map(metadata)$df
#' }
#' @importFrom ggplot2 geom_polygon scale_fill_gradient map_data
#' @importFrom dplyr left_join %>% group_by ungroup slice rowwise mutate if_else
#' @importFrom stringr str_to_title
#' @importFrom maps map
#' @export
plot_world_map <- function(metadata, base_size=8){
    long <- lat <- group <- count <- city_ascii <- region.y <- region <- ID <- date <- NULL
    if (nrow(metadata) < 1){
        error_msg <- paste("No records found in the metadata dataframe.")
        return(list(plot = NULL, df = error_msg))
    }
    if (!all(c("ID", "region", "date") %in% colnames(metadata))) {
        stop("The metadata dataframe must contain 'ID', 'region', and 'date' columns.")
    }

  
    #============= data preparation section - map city to region ========================#
    city_mapper_url <- "https://raw.githubusercontent.com/pendy05/vDiveR/main/inst/extdata/city_mapper.csv"
    city2region <- tryCatch({
        message("Loading city_mapper.csv from GitHub...")
        utils::read.csv(city_mapper_url, stringsAsFactors = FALSE)
    }, error = function(e) {
        stop("Unable to load city_mapper.csv from GitHub. Please check your internet connection.")
    })

    if (nrow(city2region) == 0) {
        stop("city_mapper.csv loaded from GitHub is empty.")
    }

    city2region$city_ascii <- tolower(city2region$city_ascii)

    # Identify duplicated city_ascii values
    duplicated_cities <- city2region %>%
        dplyr::filter(duplicated(city_ascii) | duplicated(city_ascii, fromLast = TRUE)) %>%
        dplyr::pull(city_ascii)

    # Check if any metadata$region matches the duplicated cities
    problematic_regions <- metadata$region[tolower(metadata$region) %in% duplicated_cities]

    # Issue a warning if there are matches
    if (length(problematic_regions) > 0) {
        warning_msg <- paste(
            "These 'region' values match with multiple possible world regions:\n",
            paste(unique(problematic_regions), collapse = ", ")
        )
        warning(warning_msg)
    }

    # Replace the 'region' column in metadata with the corresponding 'region' value of the matched 'city_ascii' from city_mapper
    metadata <- metadata %>%
        dplyr::mutate(
            region = tolower(region), # Convert 'region' to lowercase
            region = dplyr::if_else(
                region %in% city2region$city_ascii,  # Check if region matches any city in city_mapper
                city2region$region[match(region, city2region$city_ascii)],  # Replace with corresponding region if city matches
                region  # Otherwise, leave it as is
            )
        )

    #============ data preparation section - map data region to ggplot2 region ===========#
    world_map <- ggplot2::map_data("world")
    world_region_list <- unique(world_map$region) # Get unique regions from the world map data

    # Apply the matching function to the 'region' column
    matched_regions <- match_region_to_target(metadata$region, world_region_list)
    # Update 'region' column with matched regions where available
    metadata$region <- ifelse(is.na(matched_regions), metadata$region, matched_regions)
    # Check unique regions in the updated 'region' column
    unique_regions <- unique(metadata$region)
    # Identify any regions not in the target ggplot world map region list
    regions_not_in_target <- setdiff(unique_regions,  world_region_list)
    # Convert 'region' to factor with levels from target list
    metadata$region <- factor(metadata$region, levels =  world_region_list)

    

    if (length(regions_not_in_target) > 0) {
         warning_info <- paste('Please ensure the provided regions are part of the ggplot2 world map regions. The following regions could not be matched and need manual review:\n',
         paste(as.character(regions_not_in_target), collapse=", "), 
         '.\nYou may check the valid regions via: unique(ggplot2::map_data("world")$region)', collapse = ", ")
        warning(warning_info)
    }


    #================= plotting and tabulating section =====================#
    region_list <- data.frame(table(metadata$region))
    colnames(region_list) <- c('region','count')

    pathogens.map <- dplyr::left_join(region_list, world_map, by = "region")

    p <- ggplot(world_map, aes(x = long, y = lat, group = group)) +
        geom_polygon(fill="lightgray", colour = "#888888") + 
        geom_polygon(data = pathogens.map, aes(fill = count), color = "#888888") +
        scale_fill_gradient(low = "#FFFFFF", high = "#E63F00", name = 'Number of Sequences')  +
        theme_classic(base_size=base_size) +
        theme(plot.background = element_rect(fill = "transparent", colour = NA),
              panel.border = element_blank(), panel.grid = element_blank(),
              axis.text = element_blank(), axis.ticks = element_blank(), 
              axis.title = element_blank(), legend.position = "right")+
        labs(
            x = "Longitude", 
            y = "Latitude", 
        )

    
    colnames(region_list) <- c('Region','Number of Sequences')

    return(list(plot=p, df=region_list))

}

#' Map data region to ggplot2 region
#'
#' This function maps the provided data region to ggplot2 region.
#' @param region_column a character vector of data region
#' @param region_list a character vector of ggplot2 region
#' @param max_dist maximum distance for string matching
#' @importFrom stringdist stringdist
match_region_to_target <- function(region_column, region_list, max_dist = 3) {
    matched_regions <- sapply(region_column, function(x) {
        # Compute string distances
        distances <- stringdist::stringdist(tolower(x), tolower(region_list), method = "lv")
        min_index <- which.min(distances)
        min_dist <- distances[min_index]

        # Check if the minimum distance is within the acceptable threshold
        if (min_dist <= max_dist) {
            # Return the target list element with matching capitalization
            region_list[min_index]
        } else {
            # Return NA if no close match is found
            NA
        }
    })
    return(matched_regions)
}

#' Entropy and total variant incidence correlation plot
#'
#' This function plots the correlation between entropy and total variant incidence
#' of all the provided protein(s).
#'
#' @param df DiMA JSON converted csv file data
#' @param host number of host (1/2)
#' @param alpha any number from 0 (transparent) to 1 (opaque)
#' @param line_dot_size dot size in scatter plot
#' @param ylabel y-axis label
#' @param xlabel x-axis label
#' @param ymax maximum y-axis
#' @param ybreak y-axis breaks
#' @param base_size base font size in plot
#' @examples plot_correlation(proteins_1host)
#' @examples plot_correlation(protein_2hosts, base_size = 2, ybreak=1, ymax=10, host = 2)
#' @return A scatter plot
#' @importFrom ggplot2 ggplot geom_point aes labs scale_x_continuous scale_y_continuous theme_classic theme element_rect facet_grid
#' @importFrom grid unit
#' @importFrom dplyr vars
#' @export
plot_correlation <- function(df, 
                             host = 1 , 
                             alpha = 1/3, 
                             line_dot_size = 3, 
                             base_size = 11,
                             ylabel = "k-mer entropy (bits)\n", 
                             xlabel = "\nTotal variants (%)", 
                             ymax = ceiling(max(df$entropy)),
                             ybreak=0.5){
    totalVariants.incidence <- entropy <- NULL
    plot2<-ggplot(df)+geom_point(mapping = aes(x=totalVariants.incidence,y=entropy),alpha=alpha,size=line_dot_size)+
        labs(y = ylabel,x= xlabel)+
        scale_x_continuous(limits = c(0, 100), breaks = seq(0, 100, 20))+
        scale_y_continuous(limits = c(0, ymax), breaks = seq(0, ymax, ybreak))+
        theme_classic(base_size = base_size)+
        theme(
            panel.border = element_rect(colour = "#000000", fill=NA, linewidth=1)
        )

    if (host == 1){ #single host
        #plot the scatter plot with density
        plot2
    }else{ #multiple host
        plot2+facet_grid(rows = vars(df$host),space = "free",switch = "x")
    }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_world_map.R
\name{plot_world_map}
\alias{plot_world_map}
\title{Geographical Distribution of Sequences Plot}
\usage{
plot_world_map(metadata, base_size = 8)
}
\arguments{
\item{metadata}{a dataframe with 3 columns, 'ID', 'region', and 'date'}

\item{base_size}{word size in plot}
}
\value{
A list with 2 elements (a plot followed by a dataframe)
}
\description{
This function plots a world map and color the affected geographical region(s)
from light (lower) to dark (higher), depends on the cumulative number of sequences.
Aside from the plot, this function also returns a dataframe with 2 columns: 'Region' and 'Number of Sequences'.
The input dataframe of this function is obtainable from metadata_extraction(), with NCBI
Protein / GISAID (EpiFlu/EpiCoV/EpiPox/EpiArbo) FASTA file as input.
}
\examples{
\dontrun{
geographical_plot <- plot_world_map(metadata)$plot
geographical_df <- plot_world_map(metadata)$df
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_dynamics_proteome.R
\name{plot_dynamics_proteome}
\alias{plot_dynamics_proteome}
\title{Dynamics of Diversity Motifs (Proteome) Plot}
\usage{
plot_dynamics_proteome(
  df,
  host = 1,
  line_dot_size = 2,
  base_size = 10,
  alpha = 1/3,
  bw = "nrd0",
  adjust = 1
)
}
\arguments{
\item{df}{DiMA JSON converted csv file data}

\item{host}{number of host (1/2)}

\item{line_dot_size}{size of dot in plot}

\item{base_size}{word size in plot}

\item{alpha}{any number from 0 (transparent) to 1 (opaque)}

\item{bw}{smoothing bandwidth of violin plot (default: nrd0)}

\item{adjust}{adjust the width of violin plot (default: 1)}
}
\value{
A plot
}
\description{
This function compactly display the dynamics of diversity motifs (index and its variants: major, minor and unique)
in the form of dot plot as well as violin plot for all the provided proteins at proteome level.
}
\examples{
plot_dynamics_proteome(proteins_1host)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_extraction.R
\name{metadata_extraction}
\alias{metadata_extraction}
\title{Metadata Extraction from NCBI/GISAID (EpiFlu/EpiCoV/EpiPox/EpiArbo) FASTA file}
\usage{
metadata_extraction(file_path, source)
}
\arguments{
\item{file_path}{path of fasta file}

\item{source}{the source of fasta file, either "NCBI" or "GISAID"}
}
\value{
A dataframe that has three columns consisting ID, collected region and collected date
}
\description{
This function retrieves metadata (ID, region, date) from the input FASTA file, with the source of, either 
NCBI (with default FASTA header) or GISAID (with default FASTA header). The function will return a dataframe
that has three columns consisting ID, collected region and collected date. Records that do not have region or date
information will be excluded from the output dataframe.
}
\examples{
filepath <- system.file('extdata','GISAID_EpiCoV.faa', package = 'vDiveR')
meta_gisaid <- metadata_extraction(filepath, 'GISAID')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concat_conserved_kmer.R
\name{concat_conserved_kmer}
\alias{concat_conserved_kmer}
\title{k-mer sequences concatenation}
\usage{
concat_conserved_kmer(
  data,
  conservation_level = "HCS",
  kmer = 9,
  threshold_pct = NULL
)
}
\arguments{
\item{data}{DiMA JSON converted csv file data}

\item{conservation_level}{CCS (completely conserved) / HCS (highly conserved)}

\item{kmer}{size of the k-mer window}

\item{threshold_pct}{manually set threshold of index.incidence for HCS}
}
\value{
A list wit csv and fasta dataframes
}
\description{
This function concatenates completely ( index incidence = 100\% ) only or both 
both the completely and highly ( 90\% <= index incidence < 100\% ) conserved 
k-mer positions that overlapped at least one k-mer position or are
adjacent to each other and generate the CCS/HCS sequence in either CSV or FASTA format
}
\examples{
csv<-concat_conserved_kmer(proteins_1host)$csv
csv_2hosts<-concat_conserved_kmer(protein_2hosts, conservation_level = "CCS")$csv
fasta <- concat_conserved_kmer(protein_2hosts, conservation_level = "HCS")$fasta
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_conservation_level.R
\name{plot_conservation_level}
\alias{plot_conservation_level}
\title{Conservation Levels Distribution Plot}
\usage{
plot_conservation_level(
  df,
  protein_order = NULL,
  conservation_label = 1,
  host = 1,
  base_size = 11,
  line_dot_size = 2,
  label_size = 2.6,
  alpha = 0.6
)
}
\arguments{
\item{df}{DiMA JSON converted csv file data}

\item{protein_order}{order of proteins displayed in plot}

\item{conservation_label}{0 (partial; show present conservation labels only) or 1 (full; show ALL conservation labels) in plot}

\item{host}{number of host (1/2)}

\item{base_size}{base font size in plot}

\item{line_dot_size}{lines and dots size}

\item{label_size}{conservation labels font size}

\item{alpha}{any number from 0 (transparent) to 1 (opaque)}
}
\value{
A plot
}
\description{
This function plots conservation levels distribution of k-mer positions, which consists of
completely conserved (black) (index incidence = 100\%), highly conserved (blue)
(90\% <= index incidence < 100\%), mixed variable (green) (20\% < index incidence <= 90\%),
highly diverse (purple) (10\% < index incidence <= 20\%) and
extremely diverse (pink) (index incidence <= 10\%).
}
\examples{
plot_conservation_level(proteins_1host, conservation_label = 1,alpha=0.8, base_size = 15)
plot_conservation_level(protein_2hosts, conservation_label = 0, host=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_data.R
\docType{data}
\name{protein_2hosts}
\alias{protein_2hosts}
\title{DiMA (v5.0.9) JSON converted-CSV Output Sample 2}
\format{
A data frame with 200 rows and 17 variables:
\describe{
  \item{proteinName}{name of the protein}
  \item{position}{starting position of the aligned, overlapping k-mer window}
  \item{count}{number of k-mer sequences at the given position}
  \item{lowSupport}{k-mer position with sequences lesser than the minimum support threshold (TRUE) are considered of low support, in terms of sample size}
  \item{entropy}{level of variability at the k-mer position, with zero representing completely conserved}
  \item{indexSequence}{the predominant sequence (index motif) at the given k-mer position}
  \item{index.incidence}{the fraction (in percentage) of the index sequences at the k-mer position}
  \item{major.incidence}{the fraction (in percentage) of the major sequence (the predominant variant to the index) at the k-mer position}
  \item{minor.incidence}{the fraction (in percentage) of minor sequences (of frequency lesser than the major variant, but not singletons) at the k-mer position}
  \item{unique.incidence}{the fraction (in percentage) of unique sequences (singletons, observed only once) at the k-mer position}
  \item{totalVariants.incidence}{the fraction (in percentage) of sequences at the k-mer position that are variants to the index (includes: major, minor and unique variants)}
  \item{distinctVariant.incidence}{incidence of the distinct k-mer peptides at the k-mer position}
  \item{multiIndex}{presence of more than one index sequence of equal incidence}
  \item{host}{species name of the organism host to the virus}
  \item{highestEntropy.position}{k-mer position that has the highest entropy value}
  \item{highestEntropy}{highest entropy values observed in the studied protein}
  \item{averageEntropy}{average entropy values across all the k-mer positions}
}
}
\usage{
protein_2hosts
}
\description{
A dummy dataset with 1 protein (Core) from two hosts, human and bat
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_data.R
\docType{data}
\name{metadata}
\alias{metadata}
\title{Metadata Input Sample}
\format{
A data frame with 1000 rows and 3 variables:
\describe{
  \item{ID}{unique identifier of the sequence}
  \item{region}{geographical region of the sequence collection}
  \item{date}{collection date of the sequence}
}
}
\usage{
metadata
}
\description{
A dummy dataset that acts as an input for plot_world_map() and plot_time()
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_data.R
\docType{data}
\name{JSON_sample}
\alias{JSON_sample}
\title{DiMA (v5.0.9) JSON Output File}
\format{
A Diversity Motif Analyzer (DiMA) tool JSON file
}
\usage{
JSON_sample
}
\description{
A sample DiMA JSON Output File which acts as the input for JSON2CSV()
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/json2csv.R
\name{json2csv}
\alias{json2csv}
\title{JSON2CSV}
\usage{
json2csv(
  json_data,
  host_name = "unknown host",
  protein_name = "unknown protein"
)
}
\arguments{
\item{json_data}{DiMA JSON output dataframe}

\item{host_name}{name of the host species}

\item{protein_name}{name of the protein}
}
\value{
A dataframe which acts as input for the other functions in vDiveR package
}
\description{
This function converts DiMA (v5.0.9) JSON output file to a dataframe with 17
predefined columns which further acts as the input for other functions provided in this vDiveR package.
}
\examples{
inputdf<-json2csv(JSON_sample)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_world_map.R
\name{match_region_to_target}
\alias{match_region_to_target}
\title{Map data region to ggplot2 region}
\usage{
match_region_to_target(region_column, region_list, max_dist = 3)
}
\arguments{
\item{region_column}{a character vector of data region}

\item{region_list}{a character vector of ggplot2 region}

\item{max_dist}{maximum distance for string matching}
}
\description{
This function maps the provided data region to ggplot2 region.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_correlation.R
\name{plot_correlation}
\alias{plot_correlation}
\title{Entropy and total variant incidence correlation plot}
\usage{
plot_correlation(
  df,
  host = 1,
  alpha = 1/3,
  line_dot_size = 3,
  base_size = 11,
  ylabel = "k-mer entropy (bits)\\n",
  xlabel = "\\nTotal variants (\%)",
  ymax = ceiling(max(df$entropy)),
  ybreak = 0.5
)
}
\arguments{
\item{df}{DiMA JSON converted csv file data}

\item{host}{number of host (1/2)}

\item{alpha}{any number from 0 (transparent) to 1 (opaque)}

\item{line_dot_size}{dot size in scatter plot}

\item{base_size}{base font size in plot}

\item{ylabel}{y-axis label}

\item{xlabel}{x-axis label}

\item{ymax}{maximum y-axis}

\item{ybreak}{y-axis breaks}
}
\value{
A scatter plot
}
\description{
This function plots the correlation between entropy and total variant incidence
of all the provided protein(s).
}
\examples{
plot_correlation(proteins_1host)
plot_correlation(protein_2hosts, base_size = 2, ybreak=1, ymax=10, host = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_entropy.R
\name{plot_entropy}
\alias{plot_entropy}
\title{Entropy plot}
\usage{
plot_entropy(
  df,
  host = 1,
  protein_order = "",
  kmer_size = 9,
  ymax = 10,
  line_size = 2,
  base_size = 8,
  all = TRUE,
  highlight_zero_entropy = TRUE
)
}
\arguments{
\item{df}{DiMA JSON converted csv file data}

\item{host}{number of host (1/2)}

\item{protein_order}{order of proteins displayed in plot}

\item{kmer_size}{size of the k-mer window}

\item{ymax}{maximum y-axis}

\item{line_size}{size of the horizontal (reference) line in plot}

\item{base_size}{word size in plot}

\item{all}{plot both the entropy and total variants (pass FALSE in to plot only the entropy)}

\item{highlight_zero_entropy}{highlight region with zero entropy (default: TRUE)}
}
\value{
A plot
}
\description{
This function plot entropy (black) and total variant (red) incidence of each
k-mer position across the studied proteins and highlight region(s) with zero entropy in yellow.
k-mer position with low support is marked with a red triangle underneath the x-axis line.
}
\examples{
plot_entropy(proteins_1host)
plot_entropy(protein_2hosts, host = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_time.R
\name{plot_time}
\alias{plot_time}
\title{Time Distribution of Sequences Plot}
\usage{
plot_time(
  metadata,
  date_format = "\%Y-\%m-\%d",
  base_size = 8,
  date_break = "2 month",
  scale = "count"
)
}
\arguments{
\item{metadata}{a dataframe with 3 columns, 'ID', 'region', and 'date'}

\item{date_format}{date format of the input dataframe}

\item{base_size}{word size in plot}

\item{date_break}{date break for the scale_x_date}

\item{scale}{plot counts or log scale the data}
}
\value{
A single plot or a list with 2 elements (a plot followed by a dataframe, default)
}
\description{
This function plots the time distribution of provided sequences in the form of bar plot with 'Month' as x-axis and
'Number of Sequences' as y-axis. Aside from the plot, this function also returns a dataframe with 2 columns: 'Date' and 'Number of sequences'.
The input dataframe of this function is obtainable from metadata_extraction(), with NCBI Protein / GISAID (EpiFlu/EpiCoV/EpiPox/EpiArbo) FASTA file as input.
}
\examples{
time_plot <- plot_time(metadata, date_format="\%d/\%m/\%Y")$plot
time_df <- plot_time(metadata, date_format="\%d/\%m/\%Y")$df
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiveR-package.R
\docType{package}
\name{vDiveR-package}
\alias{vDiveR}
\alias{vDiveR-package}
\title{vDiveR: Visualization of Viral Protein Sequence Diversity Dynamics}
\description{
To ease the visualization of outputs from Diversity Motif Analyser ('DiMA'; \url{https://github.com/BVU-BILSAB/DiMA}). 'vDiveR' allows visualization of the diversity motifs (index and its variants – major, minor and unique) for elucidation of the underlying inherent dynamics. Please refer \url{https://vdiver-manual.readthedocs.io/en/latest/} for more information.
}
\author{
\strong{Maintainer}: Pendy Tok \email{pendytok0518@gmail.com}

Authors:
\itemize{
  \item Li Chuin Chong
  \item Evgenia Chikina
  \item Yin Cheng Chen
  \item Mohammad Asif Khan
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_extraction.R
\name{extract_from_NCBI}
\alias{extract_from_NCBI}
\title{Extract metadata via fasta file from ncbi}
\usage{
extract_from_NCBI(file_path)
}
\arguments{
\item{file_path}{path of fasta file}
}
\description{
This function get the metadata from each head of fasta file
}
